(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "big-uuid", "../Enums/IFIDVersions", "../TypeGuards/isAGTVersion", "../TypeGuards/isFormatMDVersion", "../TypeGuards/isIFIDVersion", "../TypeGuards/isUUIDTypeIFIDVersion", "../Enums/MagneticScrollsDocumentedTitles"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const big_uuid_1 = require("big-uuid");
    const IFIDVersions_1 = require("../Enums/IFIDVersions");
    const isAGTVersion_1 = require("../TypeGuards/isAGTVersion");
    const isFormatMDVersion_1 = require("../TypeGuards/isFormatMDVersion");
    const isIFIDVersion_1 = require("../TypeGuards/isIFIDVersion");
    const isUUIDTypeIFIDVersion_1 = require("../TypeGuards/isUUIDTypeIFIDVersion");
    const MagneticScrollsDocumentedTitles_1 = require("../Enums/MagneticScrollsDocumentedTitles");
    exports.strings = {
        AGT_SIGNATURE_INVALID: 'The agtSignature option was missing from the options object. This ' +
            'property is needed for legacy AGT IFIDs.',
        AGT_VERSION_INVALID: 'The agtVersion option was missing from the options object. This ' +
            'property is needed for legacy AGT IFIDs.',
        CHECKSUM_MISSING: 'The checksum property was missing from the options object. This ' +
            'property is needed for post-1990 Z-Code IFIDs.',
        FILE_CONTENTS_MISSING: 'An IFID version was selected requiring hashing the contents of a file, ' +
            'but no fileContents property was provided in the options object.',
        NAME_MISSING: 'The name argument was not provided in the argument object.',
        NAMESPACE_ID_MISSING: 'The namespace ID was not provided for the v3/5 UUID to be generated.',
        NO_HASHER_FOUND: 'No suitable hashing function could be found.',
        RELEASE_NUMBER_MISSING: 'The releaseNumber property was missing from the options object. This ' +
            'property is needed for Z-Code IFIDs.',
        SERIAL_CODE_MISSING: 'The serialCode property was missing from the options object. This ' +
            'property is needed for Z-Code IFIDs.',
        UNRECOGNIZED_DOCUMENTED_MAGNETIC_SCROLLS_TITLE: 'The title provided for a Magnetic Scrolls IFID was not recognized. An ' +
            'IFID should be constructed using the undocumented Magnetic Scrolls ' +
            'IFID pattern instead.',
        UUID_INVALID: 'The UUID was not properly generated for the IFID before being requested.',
        VERSION_INVALID: 'The version property of the provided argument object did not meet the ' +
            'isIFIDVersion type guard, or is not implemented.',
    };
    class IFID {
        constructor(options) {
            this.__version = IFIDVersions_1.IFIDVersions.UUID;
            this.__id = '';
            if (options) {
                if ('version' in options) {
                    if (isIFIDVersion_1.isIFIDVersion(options.version)) {
                        this.__version = options.version;
                    }
                    else {
                        throw new Error(exports.strings.VERSION_INVALID);
                    }
                }
            }
            const uuidGenerator = (() => {
                if (options && typeof options.uuidGenerator === 'function') {
                    return options.uuidGenerator;
                }
                else {
                    return (version) => (new big_uuid_1.UUID({
                        version,
                    }));
                }
            })();
            if (this.version === IFIDVersions_1.IFIDVersions.UUIDv1) {
                this.__uuid = uuidGenerator(1);
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.UUIDv4) {
                this.__uuid = uuidGenerator(4);
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.FileBasedMD5) {
                if (!options.fileContents) {
                    throw new Error(exports.strings.FILE_CONTENTS_MISSING);
                }
                const hash = big_uuid_1.cryptoJs.MD5(options.fileContents);
                this.__id = big_uuid_1.cryptoJs.hex.stringify(hash);
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.FileBasedSHA) {
                if (!options.fileContents) {
                    throw new Error(exports.strings.FILE_CONTENTS_MISSING);
                }
                const hash = big_uuid_1.cryptoJs.SHA256(options.fileContents);
                const hashed = big_uuid_1.cryptoJs.hex.stringify(hash);
                if (hashed.length > 63) {
                    this.__id = hashed.slice(0, 63);
                }
                else {
                    this.__id = hashed;
                }
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.Pre1990ZCode) {
                if (!options.releaseNumber) {
                    throw new Error(exports.strings.RELEASE_NUMBER_MISSING);
                }
                else if (!options.serialCode) {
                    throw new Error(exports.strings.SERIAL_CODE_MISSING);
                }
                this.__id = `ZCODE-${options.releaseNumber}-${options.serialCode}`;
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.Post1990ZCode) {
                if (!options.releaseNumber) {
                    throw new Error(exports.strings.RELEASE_NUMBER_MISSING);
                }
                else if (!options.serialCode) {
                    throw new Error(exports.strings.SERIAL_CODE_MISSING);
                }
                else if (!options.checksum) {
                    throw new Error(exports.strings.CHECKSUM_MISSING);
                }
                this.__id = `ZCODE-${options.releaseNumber}-${options.serialCode}-${options.checksum}`;
            }
            else if (isFormatMDVersion_1.isFormatMDVersion(this.version)) {
                if (!options.fileContents) {
                    throw new Error(exports.strings.FILE_CONTENTS_MISSING);
                }
                const hash = big_uuid_1.cryptoJs.MD5(options.fileContents);
                const hashed = big_uuid_1.cryptoJs.hex.stringify(hash);
                this.__id = `${this.version}-${hashed}`;
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.DocumentedMagneticScrolls) {
                if (!options.name) {
                    throw new Error(exports.strings.NAME_MISSING);
                }
                const format = (str) => str.replace(/[ ?!]/g, '').toLowerCase();
                const name = options.name;
                if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.ThePawn ||
                    format(name) === 'thepawn') {
                    this.__id = 'MAGNETIC-1';
                }
                else if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.GuildOfThieves ||
                    format(name) === 'guildofthieves') {
                    this.__id = 'MAGNETIC-2';
                }
                else if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.Jinxter ||
                    format(name) === 'jinxter') {
                    this.__id = 'MAGNETIC-3';
                }
                else if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.Corruption ||
                    format(name) === 'corruption') {
                    this.__id = 'MAGNETIC-4';
                }
                else if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.Fish ||
                    format(name) === 'fish') {
                    this.__id = 'MAGNETIC-5';
                }
                else if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.Myth ||
                    format(name) === 'myth') {
                    this.__id = 'MAGNETIC-6';
                }
                else if (name === MagneticScrollsDocumentedTitles_1.MagneticScrollsDocumentedTitles.Wonderland ||
                    format(name) === 'wonderland') {
                    this.__id = 'MAGNETIC-7';
                }
                else {
                    throw new Error(exports.strings.UNRECOGNIZED_DOCUMENTED_MAGNETIC_SCROLLS_TITLE);
                }
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.UndocumentedMagneticScrolls) {
                if (!options.fileContents) {
                    throw new Error(exports.strings.FILE_CONTENTS_MISSING);
                }
                const hash = big_uuid_1.cryptoJs.MD5(options.fileContents);
                const hashed = big_uuid_1.cryptoJs.hex.stringify(hash);
                this.__id = `MAGNETIC-${hashed}`;
            }
            else if (this.version === IFIDVersions_1.IFIDVersions.LegacyAGT) {
                if (!isAGTVersion_1.isAGTVersion(options.agtVersion)) {
                    throw new Error(exports.strings.AGT_VERSION_INVALID);
                }
                else if (typeof options.agtSignature !== 'string' ||
                    !options.agtSignature ||
                    options.agtSignature.length !== 8) {
                    throw new Error(exports.strings.AGT_SIGNATURE_INVALID);
                }
                const agtVer = options.agtVersion;
                let agtId = 'AGT-' +
                    (options.agtLargeOrSoggy ? agtVer.slice(0, -1) + '1' : agtVer) +
                    `-${options.agtSignature}`;
                this.__id = agtId;
            }
        }
        get version() {
            return this.__version;
        }
        get uuid() {
            return this.__uuid;
        }
        get id() {
            if (isUUIDTypeIFIDVersion_1.isUUIDTypeIFIDVersion(this.version)) {
                if (!this.uuid) {
                    throw new Error(exports.strings.UUID_INVALID);
                }
                return this.uuid.toString();
            }
            else {
                return this.__id;
            }
        }
        toString() {
            return this.id;
        }
    }
    exports.IFID = IFID;
    exports.default = IFID;
});
